﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Media.Animation;
using System.ComponentModel;
using Infragistics.Windows;
using Infragistics.Windows.DataPresenter;
using System.Diagnostics;
using System.Linq;

namespace XamDataGridSearchEngine
{
	/// <summary>
	/// Interaction logic for UserControl1.xaml
	/// </summary>
	public partial class SearchBar : UserControl
	{
        private XamDataGridEx grid;
        private List<Cell> resultRecords;
        private List<LabelPresenter> resultLabels;
        private int currentIndex;
        int currentHeaderIndex;
        private bool isOpen = false;

		public SearchBar()
		{
			this.InitializeComponent();
		}

        private void Button_Click(object sender, RoutedEventArgs e)
        {
            if (isOpen)
            {
                Storyboard closeBoard = TryFindResource("CloseSearchBar") as Storyboard;
                BeginStoryboard(closeBoard);

                searchBox.Text = "";

                isOpen = false;
            }
            else
            {
                Storyboard openBoard = TryFindResource("OpenSearchBar") as Storyboard;
                BeginStoryboard(openBoard);
                
                isOpen = true;
            }
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            resultRecords = new List<Cell>();
            resultLabels = new List<LabelPresenter>();
        }

        private void searchBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            string searchStr = (sender as TextBox).Text.ToLower();
            
        //foreach (DataRecord rec in resultRecords)
        //{
            foreach (Cell cell in resultRecords)
            {
                cell.Tag = "";
            }
        //}
            resultLabels.Clear();
            foreach (var item in grid.Labels)
            {
                item.Value.Background =  grid.Resources["labelBG"] as LinearGradientBrush;
            }

            resultRecords.Clear();

            if (searchStr.Equals(""))
            {
                grid.CellColoringBindingTrigger = !grid.CellColoringBindingTrigger;
                return; 
            }

            foreach (var item in grid.Labels)
            {
                if (item.Key.ToLower().Contains(searchStr))
                {
                    item.Value.Background = Brushes.Green;
                    resultLabels.Add(item.Value);
                }
            }

            if (grid != null && grid.Records is DataRecordCollection)
            {
                foreach (DataRecord record in grid.Records)
                {
                    //if (record.HasChildren)
                    //{
                    //    foreach (var item in record.ChildRecords[0].ChildRecords)
                    //    {
                    //        foreach (Cell cell in (item as DataRecord).Cells)
                    //        {
                    //            string cellString = cell.Value.ToString().ToLower();

                    //            if (cellString.Contains(searchStr))
                    //            {
                    //                cell.Tag = "match";
                    //                resultRecords.Add(cell);
                    //            }
                    //        }
                    //    }
                    //}
                    foreach (Cell cell in record.Cells)
                    {
                        string cellString = cell.Value.ToString().ToLower();

                        if (cellString.Contains(searchStr))
                        {
                            cell.Tag = "match";
                            resultRecords.Add(cell);
                        }
                    }

                    if (record.HasChildren)
                    {
                        AddMatchingRecords(record.ChildRecords, resultRecords, searchStr);
                    }
                }
            }

            if (grid.Records is GroupByRecordCollection)
            {
                foreach (GroupByRecord gr in grid.Records)
                {
                    AddMatchingRecords(gr, resultRecords, searchStr);
                }
            }

            if (resultRecords.Count > 0 || resultLabels.Count > 0)
            {
                currentIndex = 0;
                currentHeaderIndex = 0;
            //    grid.BringCellIntoView(resultRecords[currentIndex]);
            }
            grid.CellColoringBindingTrigger = !grid.CellColoringBindingTrigger;
            
        }

        public void AddMatchingRecords(GroupByRecord gr, List<Cell> recList, string text)
        {
            if (gr.HasChildren)
            {

                foreach (Record record in gr.ChildRecords)
                {
                    if (record is DataRecord)
                    {
                        foreach (Cell cell in (record as DataRecord).Cells)
                        {
                            string cellValue = cell.Value.ToString().ToLower();
                            if (cellValue.Contains(text))
                            {
                                cell.Tag = "match";
                                recList.Add(cell);
                            }
                        }
                    }

                    if (record is GroupByRecord)
                    {
                        AddMatchingRecords(record as GroupByRecord, recList, text);
                    }

                }
                
            }
        }

        public void AddMatchingRecords(RecordCollectionBase gr, List<Cell> recList, string text)
        {

                foreach (Record record in gr)
                {
                    if (record is DataRecord)
                    {
                        foreach (Cell cell in (record as DataRecord).Cells)
                        {
                            string cellValue = cell.Value.ToString().ToLower();
                            if (cellValue.Contains(text))
                            {
                                cell.Tag = "match";
                                recList.Add(cell);
                            }
                        }
                    }

                    if (record.HasChildren)
                    {
                        if (record is DataRecord)
                        {
                            AddMatchingRecords((record as DataRecord).ChildRecords, recList, text);   
                        }
                        if (record is ExpandableFieldRecord)
                        {
                            AddMatchingRecords((record as ExpandableFieldRecord).ChildRecords, recList, text);
                        }
                    }

            }
        }
        
        bool nxt = false;
        bool prv = false;
        private void next_Click(object sender, RoutedEventArgs e)
        {
            nxt = true;
            if (resultLabels.Count > 0 && currentHeaderIndex < resultLabels.Count)
            {
                if (currentHeaderIndex < 0)
                    currentHeaderIndex = 0;
                double width = grid.Labels.Where(f => f.Value.Field.ActualPosition.Column < resultLabels[currentHeaderIndex].Field.ActualPosition.Column).Sum(ff => ff.Value.ActualWidth);
                grid.ScrollInfo.SetHorizontalOffset(width + grid.DefaultFieldLayout.RecordSelectorExtentResolved);
            
                resultLabels[currentHeaderIndex].Background = Brushes.YellowGreen;
                foreach (var item in resultLabels.Where(f => f != resultLabels[currentHeaderIndex]))
                {
                    item.Background = Brushes.Green;
                }
                currentHeaderIndex++;
            }
            else
            {
                if (currentIndex < resultRecords.Count)
                {
                    if (resultLabels.Count > 0)
                        resultLabels[currentHeaderIndex - 1].Background = Brushes.Green;
                    grid.BringCellIntoView(resultRecords[currentIndex]);
                    grid.ActiveCell = resultRecords[currentIndex];
                    currentIndex++;

                }
                else
                {
                    MessageBox.Show("No More Results Found");
                }
            }

        }

        private void prev_Click(object sender, RoutedEventArgs e)
        {
            prv = true;
            if (currentIndex > 0)
            {
                currentIndex--;
                if (nxt)
                {
                    nxt = false;
                    currentIndex--;
                }
                grid.BringCellIntoView(resultRecords[currentIndex]);
                grid.ActiveCell = resultRecords[currentIndex];
            }
            else
            {
                if (resultLabels.Count > 0 && currentHeaderIndex >= 0)
                {
                    if (currentHeaderIndex == resultLabels.Count)
                        currentHeaderIndex--;

                    if (nxt)
                    {
                        nxt = false;
                        currentHeaderIndex--;
                    }

                    if (currentHeaderIndex < resultLabels.Count - 1)
                        resultLabels[currentHeaderIndex + 1].Background = Brushes.Green;
                    resultLabels[currentHeaderIndex].Background = Brushes.YellowGreen;

                    double width = grid.Labels.Where(f => f.Value.Field.ActualPosition.Column < resultLabels[currentHeaderIndex].Field.ActualPosition.Column).Sum(ff => ff.Value.ActualWidth);
                    grid.ScrollInfo.SetHorizontalOffset(width);
                    currentHeaderIndex--;
                }
                else
                {
                    //resultLabels[0].Background = Brushes.Green;
                    currentHeaderIndex = 0;
                    MessageBox.Show("No More Results Found");
                }
            }
        }

        private void UserControl_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.F && Keyboard.IsKeyDown(Key.LeftCtrl))
            {
                if (isOpen)
                {
                    Storyboard closeBoard = TryFindResource("CloseSearchBar") as Storyboard;
                    BeginStoryboard(closeBoard);
                    
                    searchBox.Text = "";

                    isOpen = false;
                }
                else
                {
                    Storyboard openBoard = TryFindResource("OpenSearchBar") as Storyboard;
                    BeginStoryboard(openBoard);

                    isOpen = true;
                }
            }
        }

        private void UserControl_LayoutUpdated(object sender, EventArgs e)
        {
            if (grid == null)
            {
                grid = Utilities.GetAncestorFromType(this, typeof(XamDataGridEx), false) as XamDataGridEx;
               
                if (grid != null)
                {
                    grid.KeyDown += UserControl_KeyDown;
                    Binding tagBinding = new Binding();
                    tagBinding.Source = searchBox;
                    tagBinding.Path = new PropertyPath("Text");
                    tagBinding.Mode = BindingMode.TwoWay;

                    grid.SetBinding(XamDataGrid.TagProperty, tagBinding);
                }
            }
        }
    }
}